package com.sun.org.apache.xerces.internal.dom;

import java.util.ArrayList;
import org.w3c.dom.Node;
import java.util.List;
import org.w3c.dom.NodeList;

public class DeepNodeListImpl implements NodeList
{
    protected NodeImpl rootNode;
    protected String tagName;
    protected int changes;
    protected List<Node> nodes;
    protected String nsName;
    protected boolean enableNS;
    
    public DeepNodeListImpl(NodeImpl rootNode, String tagName) {
        this.changes = 0;
        this.enableNS = false;
        this.rootNode = rootNode;
        this.tagName = tagName;
        this.nodes = new ArrayList();
    }
    
    public DeepNodeListImpl(NodeImpl rootNode, String nsName, String tagName) {
        this(rootNode, tagName);
        this.nsName = ((nsName != null && nsName.length() != 0) ? nsName : null);
        this.enableNS = true;
    }
    
    @Override
    public int getLength() {
        this.item(Integer.MAX_VALUE);
        return this.nodes.size();
    }
    
    @Override
    public Node item(int index) {
        if (this.rootNode.changes() != this.changes) {
            this.nodes = new ArrayList();
            this.changes = this.rootNode.changes();
        }
        int currentSize = this.nodes.size();
        if (index < currentSize) {
            return (Node)this.nodes.get(index);
        }
        Node thisNode;
        if (currentSize == 0) {
            thisNode = this.rootNode;
        }
        else {
            thisNode = (NodeImpl)(NodeImpl)this.nodes.get(currentSize - 1);
        }
        while (thisNode != null && index >= this.nodes.size()) {
            thisNode = this.nextMatchingElementAfter(thisNode);
            if (thisNode == null) {
                continue;
            }
            this.nodes.add(thisNode);
        }
        return thisNode;
    }
    
    protected Node nextMatchingElementAfter(Node current) {
        while (current != null) {
            if (current.hasChildNodes()) {
                current = current.getFirstChild();
            }
            else {
                Node next;
                if (current != this.rootNode && null != (next = current.getNextSibling())) {
                    current = next;
                }
                else {
                    next = null;
                    while (current != this.rootNode) {
                        next = current.getNextSibling();
                        if (next != null) {
                            break;
                        }
                        current = current.getParentNode();
                    }
                    current = next;
                }
            }
            if (current == this.rootNode || current == null || current.getNodeType() != 1) {
                continue;
            }
            if (!this.enableNS) {
                if (!this.tagName.equals("*") && !((ElementImpl)current).getTagName().equals(this.tagName)) {
                    continue;
                }
                return current;
            }
            else if (this.tagName.equals("*")) {
                if (this.nsName != null && this.nsName.equals("*")) {
                    return current;
                }
                ElementImpl el = (ElementImpl)current;
                if ((this.nsName == null && el.getNamespaceURI() == null) || (this.nsName != null && this.nsName.equals(el.getNamespaceURI()))) {
                    return current;
                }
                continue;
            }
            else {
                ElementImpl el = (ElementImpl)current;
                if (el.getLocalName() != null && el.getLocalName().equals(this.tagName)) {
                    if (this.nsName != null && this.nsName.equals("*")) {
                        return current;
                    }
                    if ((this.nsName == null && el.getNamespaceURI() == null) || (this.nsName != null && this.nsName.equals(el.getNamespaceURI()))) {
                        return current;
                    }
                    continue;
                }
                else {
                    continue;
                }
            }
        }
        return null;
    }
}
